#include "TimeName.h"

namespace {
    char ERROR_TEXT[] = "error.txt";
    char N_0TO9[] = "0123456789";
    char SERIAL_TEXT[] = "serial.txt";
    const bool PUT_NEW_LINE = true;
    const int C_SIZE_1 = C_SIZE - 1;
    const int NOT_CONVERTED = -1;
    const std::string DOC = "doc";
    const std::string HYPHEN = "-";
    const std::string DOC_ZERO = "doc-0";
    int sn;
}

TimeNames::~TimeNames() {
}

TimeNames::TimeNames(std::string & b, std::string & e) {
    base = b;
    extension = e;
    name = DOC;
    time_name = DOC_ZERO;
    readSerialNumber();
}

int TimeNames::readSerialNumber() {
    using namespace std;

    char c[C_SIZE];
    int r = sn;

    try {
        ifstream serialFile(SERIAL_TEXT);
        serialFile.exceptions(ios_base::failbit);

        if( serialFile.getline(c, C_SIZE) ) {
            string s(c);

            sn = sequence_to_integer(s);
            r = sn;
        }

    } catch(...) {
        r = zeroSerialNumber();
    }

    return r;
}

int TimeNames::sequence_to_integer(std::string & s) {
    using namespace std;

    char c[C_SIZE];
    int error_number, k, m;
    stringstream os;
    stringstream ss(s);
    ss >> k;

    if( k == 0 ) {
        c[C_SIZE_1] = '\0';
        strncpy(c, s.c_str(), C_SIZE_1);
        m = strspn(c, N_0TO9);
        if( m == 0 ) {
            k = NOT_CONVERTED;

            error_number = 221;
            os << "sequence_to_integer() error-" << error_number << ": " << s;

            MessagesTexts m(ERROR_TEXT, os.str(), PUT_NEW_LINE);
            m.setting();
            exit(error_number);
        }

    }

    return k;
}

int TimeNames::zeroSerialNumber() {
    using namespace std;

    int error_number;
    int r = sn;
    stringstream os;

    try {
        ofstream serialFile(SERIAL_TEXT);
        serialFile.exceptions(ios_base::failbit);

        sn = 0;
        serialFile << sn;

        r = sn;
    } catch(...) {
        error_number = 222;
        os << "zeroSerialNumber() error-" << error_number << ": " << SERIAL_TEXT;

        MessagesTexts m(ERROR_TEXT, os.str(), PUT_NEW_LINE);
        m.setting();
        exit(error_number);
    }

    return r;
}

std::string TimeNames::temporaryName(bool serial_number) {
    using namespace std;

    string combo;
    stringstream os;

    int minutes, seconds;
    struct tm *time_block;
    time_t timer;
    timer = time(NULL);
    time_block = localtime(&timer);
    minutes = time_block->tm_min;
    seconds = time_block->tm_sec;

    if( serial_number ) {
        incrementSerialNumber();
    }

    os << name << minutes << HYPHEN << seconds << extension;
    combo = os.str();
    if( combo == time_name ) {
        stringstream om;

        incrementSerialNumber();
        om << name << minutes << HYPHEN << seconds << extension;
        combo = om.str();
    }

    time_name = combo;
    return time_name;
}

void TimeNames::incrementSerialNumber() {
    using namespace std;

    int error_number;
    stringstream os;

    try {
        ofstream serialFile(SERIAL_TEXT);
        serialFile.exceptions(ios_base::failbit);

        sn += 1;
        serialFile << sn;
        os << base << sn << HYPHEN;
        name = os.str();
    } catch(...) {
        error_number = 223;
        os << "incrementSerialNumber() error-" << error_number << ": " << SERIAL_TEXT;

        MessagesTexts m(ERROR_TEXT, os.str(), PUT_NEW_LINE);
        m.setting();
        exit(error_number);
    }

}

