#include "parameters.h"

namespace {
    const char HYPHEN_C = '-';
    const char STANDARD_QUOTE_C = 34; // " 
    const char YEN_C = '\\';
    const std::string ASCII_1 = " !'#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
    const std::string CAPITAL_LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    const std::string EMPTY = "";
    const std::string N_0TO1 = "01";
    const std::string N_0TO9 = "0123456789";
}

bool CommandParameters::headHyphen() const {
    return head_hyphen;
}

bool CommandParameters::positivelyRepresented() const {
    return positively_represented;
}

bool CommandParameters::smallLength() const {
    return small_length;
}

bool CommandParameters::tailHyphen() const {
    return tail_hyphen;
}

bool CommandParameters::withASCII() const {
    return with_ASCII;
}

bool CommandParameters::withBinary() const {
    return with_binary;
}

bool CommandParameters::withFigure() const {
    return with_figure;
}

bool CommandParameters::withYen() const {
    return with_yen;
}

CommandParameters::~CommandParameters() {
}

CommandParameters::CommandParameters(char *a) {
    using namespace std;

    const int C_SIZE_1 = C_SIZE - 1;
    int q;
    string z(a);

    ASCII = ASCII_1 + STANDARD_QUOTE_C;
    s = z;
    s_length = s.length();
    c[C_SIZE_1] = '\0';
    strncpy(c, a, C_SIZE_1);

    head_hyphen = ( c[0] == HYPHEN_C ) ? true : false;
    small_length = ( s_length < 3 ) ? true : false;
    tail_hyphen = ( c[2] == HYPHEN_C ) ? true : false;
    with_yen = ( c[2] == YEN_C ) ? true : false;

    q = ASCII.find(c[2]);
    with_ASCII = ( q >= 0 ) ? true : false;

    q = N_0TO9.find(c[2]);
    with_figure = ( q >= 0 ) ? true : false;

    with_binary = ( s_length < 3 ) ? false : true;
    for(int i = 2; i < s_length; ++i) {
        q = N_0TO1.find(c[i]);
        if( q < 0 ) {
            with_binary = false;
            break;
        }

    }

    if( with_binary ) {
        hyphenated_character = s.substr(0, 2);

        q = CAPITAL_LETTERS.find(c[1]);
        if( q > 0 ) {
            positively_represented = false;
            negative_pattern = s.substr(2);
            positive_pattern = EMPTY;
        } else {
            positively_represented = true;
            negative_pattern = EMPTY;
            positive_pattern = s.substr(2);
        }

    } else {
        hyphenated_character = EMPTY;
        negative_pattern = EMPTY;
        positive_pattern = EMPTY;
        positively_represented = false;
    }

}

int CommandParameters::parameterLength() const {
    return s_length;
}

std::string CommandParameters::commandParameter() const {
    return s;
}

std::string CommandParameters::hyphenatedCharacter() const {
    return hyphenated_character;
}

std::string CommandParameters::negativePattern() const {
    return negative_pattern;
}

std::string CommandParameters::positivePattern() const {
    return positive_pattern;
}

bool ParameterList::character(int i) const {
    return p[i]->withASCII();
}

bool ParameterList::denied(int i) const {
    return p[i]->tailHyphen();
}

bool ParameterList::hyphenated(int i) const {
    return p[i]->headHyphen();
}

bool ParameterList::number(int i) const {
    return p[i]->withFigure();
}

bool ParameterList::pattern(int i) const {
    return p[i]->withBinary();
}

bool ParameterList::referred(int i) const {
    return p[i]->withYen();
}

bool ParameterList::shorter(int i) const {
    return p[i]->smallLength();
}

int ParameterList::itemCount() const {
    return item_count;
}

ParameterList::~ParameterList() {
    for(int i = 0; i < item_count; ++i) {
        delete p[i];
    }

}

ParameterList::ParameterList(int c, char **v) {
    item_count = c;

    for(int i = 0; i < item_count; ++i) {
        p[i] = new CommandParameters(v[i]);
    }

    for(int i = item_count; i < Q_SIZE; ++i) {
        p[i] = NULL;
    }

}

std::string ParameterList::item(int i) const {
    return p[i]->commandParameter();
}

std::string ParameterList::negative(int i) const {
    return p[i]->hyphenatedCharacter();
}

std::string ParameterList::positive(int i) const {
    return p[i]->hyphenatedCharacter();
}

std::string ParameterList::represented(int i) const {
    std::string r = p[i]->positivelyRepresented() ? p[i]->positivePattern() : p[i]->negativePattern();
    return r;
}
