#ifndef HYPHEN_TERM_H
#define HYPHEN_TERM_H

#define F_SIZE 3072
#define P_SIZE 128 // P_SIZE > Q_SIZE
#include <cmath>
#include <fstream>
#include <math.h>
#include <mem.h>
#include "Numbered.h"
#include "parameters.h"

typedef struct Triplet Triplet;
struct Triplet {
    int t[3];
};

class HyphenTerms {
    bool binary_represented, carriage_return;
    bool deniable[13];
    char CLEAR_BIT_C, GAP_C, SET_BIT_C, TAB_C;
    char buffer[F_SIZE];
    char c[C_SIZE];
    int after, before, field_count, focal_point, interval_from, interval_to;
    int byteCode[5];
    int figure[7];
    int maximum[3];
    int minimum[3];
    ParameterList *listing;
    std::string ASCII, BYTES, EBB, FLOOD, first_file_name, joint_piece, label_piece, NEW_LINE, path_name, pattern, quote_piece, second_file_name, separator_piece;
    std::string acquisition[Q_SIZE];
    std::string bit_pattern_array[3];
    std::string commandParameter[P_SIZE];
    std::string list_name[3];
    std::string piece[5];
    Triplet triplet_in_context;
    unsigned char joint_byte, label_byte, quote_byte, separator_byte;
public:
    ~HyphenTerms();
    HyphenTerms(ParameterList *L); // presetting
    bool binaryRepresented() const;
    bool carriageReturn();
    bool checkLength();
    bool countLines();
    bool extensionEqualsCSV(std::string & file_name);
    bool guideFile();
    bool include0Dh() const;
    bool identicalTime();
    bool jointPieces(std::string & s);
    bool labelPieces(std::string & s);
    bool lineFeed();
    bool noRemoving();
    bool produceTemporaryBinary();
    bool quotePieces(std::string & s);
    bool remove0Dh();
    bool separatorPieces(std::string & s);
    bool versionNumber();
    bool withBOM();
    int acquireFromText();
    int afterPoint();
    int atPoint();
    int beforePoint();
    int copyingStringToArray(char *s, int s_length);
    int hexadecimal_to_integer(std::string & s);
    int intervalFrom() const;
    int intervalTo() const;
    int tripletAfter();
    int tripletBefore();
    int tripletPoint();
    std::string fileName() const;
    std::string jointPiece();
    std::string joints();
    std::string labelPiece();
    std::string labels();
    std::string listName(int index);
    std::string multipleBitPattern();
    std::string negativePattern();
    std::string newName();
    std::string pathName() const;
    std::string quotePiece();
    std::string quotes();
    std::string separatorPiece();
    std::string separators();
    std::string singleSurge(); // not Triplet 
    std::string surge_of_set_bits(int interval_0, int interval_1);
    Triplet headEnd();
    Triplet inContext();
    Triplet nearPoint();
    Triplet tailEnd();
    Triplet tailEnd(int line_count);
    unsigned char jointByte() const;
    unsigned char jointCode();
    unsigned char labelByte() const;
    unsigned char labelCode();
    unsigned char punctuation(std::string & s, int q);
    unsigned char quoteByte() const;
    unsigned char quoteCode();
    unsigned char separatorByte() const;
    unsigned char separatorCode();
    void setFieldCount(int c);
    void setting();
};

#endif // HYPHEN_TERM_H
