#ifndef PRODUCE_UTF16_H
#define PRODUCE_UTF16_H

/*
Utf8to16 
Producing a UTF-16 text from the UTF-8 text
Copyright 2019-2020 Sogaya

Borland C++ 5.5.1 for Win32 (BCC55)
Digital Mars Compiler Version 8.42n (DMC 8.57 [dm857c.zip] ), STLport 4.5.3
Embarcadero C++ 7.20 for Win32 (BCC101)
Embarcadero C++ 7.30 for Win32 (BCC102)
GCC 3.4.2 (mingw-special [Bloodshed Dev-C++ 4.9.9.2] )

*/
#define B_SIZE 21
#define C_SIZE 256
#define J_SIZE 8
#define P_SIZE 128 // P_SIZE > Q_SIZE
#define V_SIZE 8192
#define W_SIZE 24576
#define Y_SIZE 4096
#include <stdlib.h>
#include "ErrorText.h"

typedef struct AsArray AsArray;
struct AsArray {
    char v[W_SIZE];
    int v_length;
};

class CodePoint {
    AsArray a;
    bool carriage_return_appears, unknown_mode, with_BOM;
    char *buffer;
    char *combo;
    char p[P_SIZE];
    char y[Y_SIZE];
    int B_110110_S, B_110111_S, beginning, buffer_size, combo_size, count, group_part_mode, higher_part_of_kanji, lower_part_of_kanji, LOWER_PART_MASK, NEW_LINE_SIZE, y_length;
    int B[B_SIZE];
    int E[C_SIZE];
    int H[J_SIZE];
    int pattern[B_SIZE];
    std::string file_name, NEW_LINE, new_name;

    int bits_of_KANJI_PART_1(int k);
    int bits_of_KANJI_PART_2(int k);
    int bits_of_KANJI_PART_3(char *p, int k, int m);
    int bits_of_SURROGATE_PAIR_PART_1(int k);
    int bits_of_SURROGATE_PAIR_PART_2(int k);
    int bits_of_SURROGATE_PAIR_PART_3(int k);
    int bits_of_SURROGATE_PAIR_PART_4(char *p, int k, int m);
    int bits_of_TWO_BYTE_CHARACTER_PART_1(int k);
    int bits_of_TWO_BYTE_CHARACTER_PART_2(char *p, int k, int m);
    int bits_of_WHOLE_ASCII(char *p, int k, int m);
    int checkGroupPartMode(int k);
    int E_table();
    int setBufferSize();
    long file_size(std::string & name);

public:
    ~CodePoint();
    CodePoint(std::string fn);
    CodePoint(std::string fn, bool wb);

    AsArray produce_UTF16_phrase(char *s, int s_length);
    AsArray reading_to_produce_UTF16();
    bool setting();
    bool setting(int bs);
    char copyingArrayItem(int index);
    int bufferingWriting(int comboLength);
    int lineCount() const;
    int produceBinaryFile(std::string & name, bool no_removing);
    int reading_to_produce_UTF16_CRLF(std::string & nn);
    int reading_to_produce_UTF16_LF(std::string & nn);
    int reading_to_produce_UTF16_text(std::string & nn);
    int renameBinaryFile(std::string & name, bool no_removing);
    std::string newLine() const;
    void ifErrorOccurs(int exit_code);
};

#endif // PRODUCE_UTF16_H
