#include "ReadWrite.h"

namespace {
    char BOM[2] = {255, 254}; // FFh, FEh
    char EOL[2] = {10, 0}; // 0Ah
    char NEW_LINE[4] = {13, 0, 10, 0}; // 0Ah
    char TEMPORARY_BINARY[] = "binary.tmp";
    const bool PUT_NEW_LINE = true;
    const bool REMOVE_ANCIENT_FILE = false;
    const char LF_C = 10; // 0Ah
    const char SENTINEL = 255; // FFh
    const int BOM_LENGTH = 2;
    const int EOL_LENGTH = 1;
    const int NEW_LINE_LENGTH = 4;
    const int NO_EXTRA_BUFFER = 0;
    const int REMOVED = -5;
    const int V_SIZE_1 = V_SIZE - 1;
    const std::string C_ERR = "err";
    const std::string EMPTY = "";
    const std::string TEXT_FILE = ".txt";
}

ReadingWriting::~ReadingWriting() {
    delete c;
}

ReadingWriting::ReadingWriting(FinalEffects *final_effects, char *nn) {
    f = final_effects;
    with_BOM = f->withBOM();
    file_name = f->fileName();
    strcpy(new_name, nn);

    c = new CodePoint(file_name, with_BOM);
    c->setting(NO_EXTRA_BUFFER);
}

bool ReadingWriting::setting() {
    using namespace std;

    bool software_can_open_file;
    int error_number;
    stringstream os;

    try {
        ifstream testFile( file_name.c_str(), ios::binary );
        testFile.exceptions(ios_base::failbit);
        software_can_open_file = true;
    } catch(...) {
        software_can_open_file = false;
        error_number = 209;
        os << "ReadingWriting::setting() error-" << error_number << ": " << file_name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    if( f->identicalTime() ) {
        c->produceBinaryFile(file_name, REMOVE_ANCIENT_FILE);
        c->renameBinaryFile( file_name, f->noRemoving() );
    }

    return software_can_open_file;
}

int ReadingWriting::checking_length_to_produce_UTF16_CRLF() {
    char *p;
    FILE *newFile = fopen(new_name, "wb");
    FILE *utf8File = fopen( file_name.c_str(), "rt");

    if( with_BOM ) {
        fwrite(BOM, BOM_LENGTH, 1, newFile);
    }

    count = 0;
    for(;;) {
        v[V_SIZE_1] = SENTINEL;

        p = fgets(v, V_SIZE, utf8File);
        if(p == NULL) {
            break;
        } else {
            p = strchr(v, '\n');
            if(p != NULL) {
                *p = '\0';
            }

        }

        count += 1;
        if( v[V_SIZE_1] == '\0' ) {
            fprintf(stderr, "Warning: full buffer ( Line-%d )\n", count);
        }

        a = c->produce_UTF16_phrase(v, strlen(v) );
        fwrite(a.v, a.v_length, 1, newFile);
        fwrite(NEW_LINE, NEW_LINE_LENGTH, 1, newFile);
    }

    fclose(utf8File);
    fclose(newFile);
    return count;
}

int ReadingWriting::checking_length_to_produce_UTF16_LF() {
    char *p;
    FILE *newFile = fopen(new_name, "wb");
    FILE *utf8File = fopen( file_name.c_str(), "rb");

    if( with_BOM ) {
        fwrite(BOM, BOM_LENGTH, 1, newFile);
    }

    count = 0;
    for(;;) {
        v[V_SIZE_1] = SENTINEL;

        p = fgets(v, V_SIZE, utf8File);
        if(p == NULL) {
            break;
        } else {
            p = strchr(v, '\r');
            if(p != NULL) {
                *p = '\0';
            }

        }

        count += 1;
        if( v[V_SIZE_1] == '\0' ) {
            fprintf(stderr, "Warning: full buffer ( Line-%d )\n", count);
        }

        a = c->produce_UTF16_phrase(v, strlen(v) );
        fwrite(a.v, a.v_length, 1, newFile);
        fwrite(EOL, EOL_LENGTH, 1, newFile);
    }

    fclose(utf8File);
    fclose(newFile);
    return count;
}

int ReadingWriting::countLines(std::string & name) {
    using namespace std;

    int block_has_been_read, count, error_number;
    stringstream os;

    try {
        ifstream testFile( name.c_str() );
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 210;
        os << "countLines() error-" << error_number << ": " << name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    ifstream nameFile( name.c_str(), ios::binary);
    count = 0;
    for(;;) {
        nameFile.read(y, Y_SIZE);
        block_has_been_read = nameFile.gcount();
        if( block_has_been_read == 0 ) {
            break;
        }

        for(int i = 0; i < block_has_been_read; ++i) {
            if( y[i] == LF_C ) {
                count += 1;
            }

        }

    }

    return count;
}

int ReadingWriting::lineCount() const {
    return count;
}

int ReadingWriting::reading_to_produce_UTF16_CRLF() {
    char *p;
    FILE *newFile = fopen(new_name, "wt");
    FILE *utf8File = fopen( file_name.c_str(), "rt");

    if( with_BOM ) {
        fwrite(BOM, BOM_LENGTH, 1, newFile);
    }

    count = 0;
    for(;;) {
        p = fgets(v, V_SIZE, utf8File);
        if(p == NULL) {
            break;
        } else {
            p = strchr(v, '\n');
            if(p != NULL) {
                *p = '\0';
            }

        }

        count += 1;
        a = c->produce_UTF16_phrase(v, strlen(v) );
        fwrite(a.v, a.v_length, 1, newFile);
        fwrite(NEW_LINE, NEW_LINE_LENGTH, 1, newFile);
    }

    fclose(utf8File);
    fclose(newFile);
    return count;
}

int ReadingWriting::reading_to_produce_UTF16_LF() {
    char *p;
    FILE *newFile = fopen(new_name, "wb");
    FILE *utf8File = fopen( file_name.c_str(), "rb");

    if( with_BOM ) {
        fwrite(BOM, BOM_LENGTH, 1, newFile);
    }

    count = 0;
    for(;;) {
        p = fgets(v, V_SIZE, utf8File);
        if(p == NULL) {
            break;
        } else {
            p = strchr(v, '\r');
            if(p != NULL) {
                *p = '\0';
            }

        }

        count += 1;
        a = c->produce_UTF16_phrase(v, strlen(v) );
        fwrite(a.v, a.v_length, 1, newFile);
        fwrite(EOL, EOL_LENGTH, 1, newFile);
    }

    fclose(utf8File);
    fclose(newFile);
    return count;
}

int ReadingWriting::renameBinaryFile(std::string & name, bool no_removing) {
    using namespace std;

    char c[C_SIZE];
    int block_has_been_read, error_number;
    int sum = 0;
    stringstream os;

    try {
        ifstream testFile(TEMPORARY_BINARY);
        testFile.exceptions(ios_base::failbit);
    } catch(...) {
        error_number = 211;
        os << "renameBinaryFile() error-" << error_number << ": " << name;

        ErrorTexts x(C_ERR, TEXT_FILE, os.str(), PUT_NEW_LINE);
        x.setting();
        exit(error_number);
    }

    if( no_removing ) {
        ifstream nameFile(TEMPORARY_BINARY, ios::binary);
        ofstream newFile( name.c_str(), ios::binary );

        for(;;) {
            nameFile.read(y, Y_SIZE);
            block_has_been_read = nameFile.gcount();
            if( block_has_been_read == 0 ) {
                break;
            }

            newFile.write(y, block_has_been_read);
            sum += block_has_been_read;
        }

    } else {
        strcpy(c, name.c_str() );
        if( rename(TEMPORARY_BINARY, c) ) {
            remove(c);
            sum = REMOVED;
            rename(TEMPORARY_BINARY, c);
        }

    }

    return sum;
}

